@@ -5,6 +5,7 @@ setClass("attrData", representation(data="list",
 setClass("graph", representation(edgemode="character",
                                  edgeData="attrData",
                                  nodeData="attrData",
+                                 graphData="list",
                                  "VIRTUAL"))
 
 ## Node Edge List representation
@@ -48,3 +49,27 @@ setClass("simpleEdge",
                         eNode="character"),   ##end   - if directed
          prototype=list(edgeType="unknown",
            directed=FALSE, bNode="", eNode="", weight=1))
+
+
+
+##multigraphs - not clear if we should extend graph here, or have a
+##whole new set of classes
+
+##looks like redefining edgeSets is going to help us out here.
+
+setClass("edgeSet", contains="VIRTUAL",
+         representation=representation(
+           edgeData="attrData"))
+
+setClass("edgeSetNEL", contains="edgeSet",
+         representation(edgemode="character", edgeL="list"))
+
+setClass("edgeSetAM", contains="edgeSet",
+         representation(edgemode="character", adjMat="matrix"))
+
+setClass("multiGraph",
+         representation(nodes="vector", edgeL="list",
+                        nodeData="attrData",
+                        graphData="list"))
+
+
