@@ -28,7 +28,7 @@ setMethod("numEdges", signature(object="graph"),
           function(object) {
               gEdges <- edges(object)
               if (length(gEdges) == 0)
-                return(length(gEdges))
+                return(0)
               numEdges <- length(unlist(gEdges, use.names=FALSE))
               if (!isDirected(object)) {
                   numSelfLoops <- sum(mapply(function(e, n) sum(n == e),
@@ -76,43 +76,43 @@ setMethod("isAdjacent",signature(object="graph", from="character",
   ##handle directed graphs by a list inDegree and outDegree
   setMethod("degree", signature(object="graph", Nodes="missing"),
       function(object)  {
-          ns <- nodes(object)
-          nl <- edges(object)
-          deg <- sapply(nl, length)
-          names(deg) <- ns
-          if( object@edgemode == "undirected" )
-              return(deg)
-          else if( object@edgemode == "directed" ) {
-              b1<- unlist(nl)
-              b2<- table(b1)
-              inDegree <- rep(0, length(ns))
-              names(inDegree) <- ns
-              inDegree[names(b2)]<-b2
-              return(list(inDegree=inDegree, outDegree=deg))
-          }
-          stop(paste("edgemode", object@edgemode, "is not valid"))
+          degree(object, Nodes=nodes(object))
      })
 
   setMethod("degree", "graph",  function(object, Nodes) {
        nl <- edges(object)
        nls <- nl[Nodes]
 
-       deg <- sapply(nls, length)
+       deg <- listLen(nls)
        names(deg) <- Nodes
-       if( object@edgemode == "undirected" )
+       if (!isDirected(object))
            return(deg)
-       else if( object@edgemode == "directed" ) {
+       else {
            b1 <- unlist(nl)
-           b2 <- table(b1)[Nodes]
-           inDegree <- rep(0, length(nls))
-           names(inDegree) <- Nodes
-           inDegree[names(b2)] <- b2
+           b2 <- table(b1)
+           nonZeroNodes <- Nodes[Nodes %in% names(b2)]
+           inDegree <- structure(integer(length(Nodes)), names=Nodes)
+           inDegree[nonZeroNodes] <- as.integer(b2[nonZeroNodes])
            return(list(inDegree=inDegree, outDegree=deg))
        }
-       stop(paste("edgemode", object@edgemode, "is not valid"))
-     })
-
+  })
 
+setMethod("leaves", "graph",
+          function(object, degree.dir) {
+              deg <- degree(object)
+              leaf_degree <- 1L
+              if (isDirected(object)) {
+                  if (missing(degree.dir))
+                    stop("'degree.dir' must be specified for a directed graph")
+                  degree.dir <- switch(match.arg(degree.dir, c("in", "out")),
+                                       "in"="inDegree",
+                                       "out"="outDegree")
+                  deg <- deg[[degree.dir]]
+                  leaf_degree <- 0L
+              }
+              wh <- deg == leaf_degree
+              names(deg)[wh]
+          })
 
 ##  setMethod("acc", "graph", function(object, index) {
 ##       visit <- function(ind) {
